@@ -726,104 +726,6 @@ eel_remove_weak_pointer (gpointer pointer_location)
 	*object_location = NULL;
 }
 
-static void
-update_auto_boolean (GSettings   *settings,
-		     const gchar *key,
-		     gpointer     user_data)
-{
-	int *storage = user_data;
-
-	*storage = g_settings_get_boolean (settings, key);
-}
-
-void
-eel_g_settings_add_auto_boolean (GSettings *settings,
-				 const char *key,
-				 gboolean *storage)
-{
-	char *signal;
-
-	*storage = g_settings_get_boolean (settings, key);
-	signal = g_strconcat ("changed::", key, NULL);
-	g_signal_connect (settings, signal,
-			  G_CALLBACK(update_auto_boolean),
-			  storage);
-}
-
-static void
-update_auto_int (GSettings   *settings,
-		 const gchar *key,
-		 gpointer     user_data)
-{
-	int *storage = user_data;
-
-	*storage = g_settings_get_int (settings, key);
-}
-
-void
-eel_g_settings_add_auto_int (GSettings *settings,
-			     const char *key,
-			     int *storage)
-{
-	char *signal;
-
-	*storage = g_settings_get_int (settings, key);
-	signal = g_strconcat ("changed::", key, NULL);
-	g_signal_connect (settings, signal,
-			  G_CALLBACK(update_auto_int),
-			  storage);
-}
-
-static void
-update_auto_enum (GSettings   *settings,
-		  const gchar *key,
-		  gpointer     user_data)
-{
-	int *storage = user_data;
-
-	*storage = g_settings_get_enum (settings, key);
-}
-
-void
-eel_g_settings_add_auto_enum (GSettings *settings,
-			      const char *key,
-			      int *storage)
-{
-	char *signal;
-
-	*storage = g_settings_get_enum (settings, key);
-	signal = g_strconcat ("changed::", key, NULL);
-	g_signal_connect (settings, signal,
-			  G_CALLBACK(update_auto_enum),
-			  storage);
-}
-
-static void
-update_auto_strv (GSettings   *settings,
-		  const gchar *key,
-		  gpointer     user_data)
-{
-	char ***storage = user_data;
-
-	g_free (*storage);
-	*storage = g_settings_get_strv (settings, key);
-}
-
-void
-eel_g_settings_add_auto_strv (GSettings *settings,
-			      const char *key,
-			      char ***storage)
-{
-	char *signal;
-
-	*storage = NULL;
-	update_auto_strv (settings, key, storage);
-	signal = g_strconcat ("changed::", key, NULL);
-	g_signal_connect (settings, signal,
-			  G_CALLBACK(update_auto_strv),
-			  storage);
-}
-
 static void
 update_auto_strv_as_quarks (GSettings   *settings,
 			    const gchar *key,
@@ -859,6 +761,8 @@ eel_g_settings_add_auto_strv_as_quarks (GSettings *settings,
 	g_signal_connect (settings, signal,
 			  G_CALLBACK(update_auto_strv_as_quarks),
 			  storage);
+
+	g_free (signal);
 }
 
 #if !defined (EEL_OMIT_SELF_CHECK)
