@@ -39,13 +39,14 @@ void nautilus_launch_application                 (GAppInfo
 void nautilus_launch_application_by_uri          (GAppInfo                          *application,
 						  GList                             *uris,
 						  GtkWindow                         *parent_window);
+void nautilus_launch_application_for_mount       (GAppInfo                          *app_info,
+						  GMount                            *mount,
+						  GtkWindow                         *parent_window);
 void nautilus_launch_application_from_command    (GdkScreen                         *screen,
-						  const char                        *name,
 						  const char                        *command_string,
 						  gboolean                           use_terminal,
 						  ...) G_GNUC_NULL_TERMINATED;
 void nautilus_launch_application_from_command_array (GdkScreen                         *screen,
-						     const char                        *name,
 						     const char                        *command_string,
 						     gboolean                           use_terminal,
 						     const char * const *               parameters);
