@@ -39,11 +39,9 @@
 #include "nautilus-icon-view.h"
 #include "nautilus-image-properties-page.h"
 #include "nautilus-list-view.h"
-#include "nautilus-navigation-window.h"
-#include "nautilus-navigation-window-slot.h"
 #include "nautilus-progress-ui-handler.h"
 #include "nautilus-self-check-functions.h"
-#include "nautilus-spatial-window.h"
+#include "nautilus-window.h"
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-private.h"
@@ -91,9 +89,6 @@ static NautilusApplication *singleton = NULL;
 /* Keeps track of all the desktop windows. */
 static GList *nautilus_application_desktop_windows;
 
-/* Keeps track of all the object windows */
-static GList *nautilus_application_spatial_window_list;
-
 /* The saving of the accelerator map was requested  */
 static gboolean save_of_accel_map_requested = FALSE;
 
@@ -109,19 +104,11 @@ G_DEFINE_TYPE (NautilusApplication, nautilus_application, GTK_TYPE_APPLICATION);
 
 struct _NautilusApplicationPriv {
 	GVolumeMonitor *volume_monitor;
-	GDBusProxy *ck_proxy;
-	gboolean session_is_active;
 	NautilusProgressUIHandler *progress_handler;
 
 	gboolean initialized;
 };
 
-static GList *
-nautilus_application_get_spatial_window_list (void)
-{
-	return nautilus_application_spatial_window_list;
-}
-
 static gboolean
 check_required_directories (NautilusApplication *application)
 {
@@ -272,150 +259,6 @@ mark_desktop_files_trusted (void)
 	g_free (do_once_file);
 }
 
-#define CK_NAME       "org.freedesktop.ConsoleKit"
-#define CK_PATH       "/org/freedesktop/ConsoleKit"
-#define CK_INTERFACE  "org.freedesktop.ConsoleKit"
-
-static void
-ck_session_proxy_signal_cb (GDBusProxy *proxy,
-			    const char *sender_name,
-			    const char *signal_name,
-			    GVariant   *parameters,
-			    gpointer    user_data)
-{
-	NautilusApplication *application = user_data;
-
-	if (g_strcmp0 (signal_name, "ActiveChanged") == 0) {
-		g_variant_get (parameters, "(b)", &application->priv->session_is_active);
-		DEBUG ("ConsoleKit session is active %d", application->priv->session_is_active);
-	}
-}
-
-static void
-ck_call_is_active_cb (GDBusProxy   *proxy,
-		      GAsyncResult *result,
-		      gpointer      user_data)
-{
-	NautilusApplication *application = user_data;
-	GVariant *variant;
-	GError *error = NULL;
-
-	variant = g_dbus_proxy_call_finish (proxy, result, &error);
-
-	if (variant == NULL) {
-		g_warning ("Error when calling IsActive(): %s\n", error->message);
-		application->priv->session_is_active = TRUE;
-
-		g_error_free (error);
-		return;
-	}
-
-	g_variant_get (variant, "(b)", &application->priv->session_is_active);
-	DEBUG ("ConsoleKit session is active %d", application->priv->session_is_active);
-
-	g_variant_unref (variant);
-}
-
-static void
-session_proxy_appeared (GObject       *source,
-                        GAsyncResult *res,
-                        gpointer      user_data)
-{
-	NautilusApplication *application = user_data;
-        GDBusProxy *proxy;
-	GError *error = NULL;
-
-        proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
-
-	if (error != NULL) {
-		g_warning ("Failed to get the current CK session: %s", error->message);
-		g_error_free (error);
-
-		application->priv->session_is_active = TRUE;
-		return;
-	}
-
-	g_signal_connect (proxy, "g-signal",
-			  G_CALLBACK (ck_session_proxy_signal_cb),
-			  application);
-
-	g_dbus_proxy_call (proxy,
-			   "IsActive",
-			   g_variant_new ("()"),
-			   G_DBUS_CALL_FLAGS_NONE,
-			   -1,
-			   NULL,
-			   (GAsyncReadyCallback) ck_call_is_active_cb,
-			   application);
-
-        application->priv->ck_proxy = proxy;
-}
-
-static void
-ck_get_current_session_cb (GDBusConnection *connection,
-			   GAsyncResult    *result,
-			   gpointer         user_data)
-{
-	NautilusApplication *application = user_data;
-	GVariant *variant;
-	const char *session_path = NULL;
-	GError *error = NULL;
-
-	variant = g_dbus_connection_call_finish (connection, result, &error);
-
-	if (variant == NULL) {
-		g_warning ("Failed to get the current CK session: %s", error->message);
-		g_error_free (error);
-
-		application->priv->session_is_active = TRUE;
-		return;
-	}
-
-	g_variant_get (variant, "(&o)", &session_path);
-
-	DEBUG ("Found ConsoleKit session at path %s", session_path);
-
-	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
-				  G_DBUS_PROXY_FLAGS_NONE,
-				  NULL,
-				  CK_NAME,
-				  session_path,
-				  CK_INTERFACE ".Session",
-				  NULL,
-				  session_proxy_appeared,
-				  application);
-
-	g_variant_unref (variant);
-}
-
-static void
-do_initialize_consolekit (NautilusApplication *application)
-{
-	GDBusConnection *connection;
-
-	connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
-
-	if (connection == NULL) {
-		application->priv->session_is_active = TRUE;
-		return;
-	}
-
-	g_dbus_connection_call (connection,
-				CK_NAME,
-				CK_PATH "/Manager",
-				CK_INTERFACE ".Manager",
-				"GetCurrentSession",
-				g_variant_new ("()"),
-				G_VARIANT_TYPE ("(o)"),
-				G_DBUS_CALL_FLAGS_NONE,
-				-1,
-				NULL,
-				(GAsyncReadyCallback) ck_get_current_session_cb,
-				application);
-
-	g_object_unref (connection);
-}
-
 static void
 do_upgrades_once (NautilusApplication *application,
 		  gboolean no_desktop)
@@ -488,9 +331,6 @@ finish_startup (NautilusApplication *application,
 	/* Initialize the desktop link monitor singleton */
 	nautilus_desktop_link_monitor_get ();
 
-	/* Initialize the ConsoleKit listener for active session */
-	do_initialize_consolekit (application);
-
 	/* Initialize the UI handler singleton for file operations */
 	notify_init (GETTEXT_PACKAGE);
 	application->priv->progress_handler = nautilus_progress_ui_handler_new ();
@@ -507,12 +347,10 @@ finish_startup (NautilusApplication *application,
 static void
 open_window (NautilusApplication *application,
 	     const char *startup_id,
-	     const char *uri, GdkScreen *screen, const char *geometry,
-	     gboolean browser_window)
+	     const char *uri, GdkScreen *screen, const char *geometry)
 {
 	GFile *location;
 	NautilusWindow *window;
-	gboolean open_in_browser;
 
 	if (uri == NULL) {
 		location = g_file_new_for_path (g_get_home_dir ());
@@ -520,24 +358,11 @@ open_window (NautilusApplication *application,
 		location = g_file_new_for_uri (uri);
 	}
 
-	open_in_browser = browser_window ||
-		g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
-
-	DEBUG ("Opening new window at uri %s, browser %d", uri, open_in_browser);
-
-	if (open_in_browser) {
-		window = nautilus_application_create_navigation_window (application,
-									startup_id,
-									screen);
-	} else {
-		window = nautilus_application_get_spatial_window (application,
-								  NULL,
-								  startup_id,
-								  location,
-								  screen,
-								  NULL);
-	}
+	DEBUG ("Opening new window at uri %s", uri);
 
+	window = nautilus_application_create_window (application,
+						     startup_id,
+						     screen);
 	nautilus_window_go_to (window, location);
 
 	g_object_unref (location);
@@ -560,18 +385,17 @@ open_windows (NautilusApplication *application,
 	      const char *startup_id,
 	      char **uris,
 	      GdkScreen *screen,
-	      const char *geometry,
-	      gboolean browser_window)
+	      const char *geometry)
 {
 	guint i;
 
 	if (uris == NULL || uris[0] == NULL) {
 		/* Open a window pointing at the default location. */
-		open_window (application, startup_id, NULL, screen, geometry, browser_window);
+		open_window (application, startup_id, NULL, screen, geometry);
 	} else {
 		/* Open windows at each requested location. */
 		for (i = 0; uris[i] != NULL; i++) {
-			open_window (application, startup_id, uris[i], screen, geometry, browser_window);
+			open_window (application, startup_id, uris[i], screen, geometry);
 		}
 	}
 }
@@ -732,7 +556,7 @@ nautilus_application_close_desktop (void)
 }
 
 void
-nautilus_application_close_all_navigation_windows (NautilusApplication *self)
+nautilus_application_close_all_windows (NautilusApplication *self)
 {
 	GList *list_copy;
 	GList *l;
@@ -743,144 +567,14 @@ nautilus_application_close_all_navigation_windows (NautilusApplication *self)
 		NautilusWindow *window;
 		
 		window = NAUTILUS_WINDOW (l->data);
-
-		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-			gtk_widget_hide (GTK_WIDGET (window));
-		}
+		gtk_widget_hide (GTK_WIDGET (window));
 	}
 
 	for (l = list_copy; l != NULL; l = l->next) {
 		NautilusWindow *window;
 		
 		window = NAUTILUS_WINDOW (l->data);
-		
-		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-			nautilus_window_close (window);
-		}
-	}
-	g_list_free (list_copy);
-}
-
-static NautilusSpatialWindow *
-nautilus_application_get_existing_spatial_window (GFile *location)
-{
-	GList *l;
-	NautilusWindowSlot *slot;
-	GFile *window_location;
-
-	for (l = nautilus_application_get_spatial_window_list ();
-	     l != NULL; l = l->next) {
-		slot = NAUTILUS_WINDOW (l->data)->details->active_pane->active_slot;
-
-		window_location = slot->pending_location;
-		
-		if (window_location == NULL) {
-			window_location = slot->location;
-		}
-
-		if (window_location != NULL) {
-			if (g_file_equal (location, window_location)) {
-				return NAUTILUS_SPATIAL_WINDOW (l->data);
-			}
-		}
-	}
-
-	return NULL;
-}
-
-static NautilusSpatialWindow *
-find_parent_spatial_window (NautilusSpatialWindow *window)
-{
-	NautilusFile *file;
-	NautilusFile *parent_file;
-	NautilusWindowSlot *slot;
-	GFile *location;
-
-	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
-
-	location = slot->location;
-	if (location == NULL) {
-		return NULL;
-	}
-	file = nautilus_file_get (location);
-
-	if (!file) {
-		return NULL;
-	}
-
-	parent_file = nautilus_file_get_parent (file);
-	nautilus_file_unref (file);
-	while (parent_file) {
-		NautilusSpatialWindow *parent_window;
-
-		location = nautilus_file_get_location (parent_file);
-		parent_window = nautilus_application_get_existing_spatial_window (location);
-		g_object_unref (location);
-
-		/* Stop at the desktop directory if it's not explicitely opened
-		 * in a spatial window of its own.
-		 */
-		if (nautilus_file_is_desktop_directory (parent_file) && !parent_window) {
-			nautilus_file_unref (parent_file);
-			return NULL;
-		}
-
-		if (parent_window) {
-			nautilus_file_unref (parent_file);
-			return parent_window;
-		}
-		file = parent_file;
-		parent_file = nautilus_file_get_parent (file);
-		nautilus_file_unref (file);
-	}
-
-	return NULL;
-}
-
-void
-nautilus_application_close_parent_windows (NautilusSpatialWindow *window)
-{
-	NautilusSpatialWindow *parent_window;
-	NautilusSpatialWindow *new_parent_window;
-
-	g_return_if_fail (NAUTILUS_IS_SPATIAL_WINDOW (window));
-
-	parent_window = find_parent_spatial_window (window);
-	
-	while (parent_window) {
-		
-		new_parent_window = find_parent_spatial_window (parent_window);
-		nautilus_window_close (NAUTILUS_WINDOW (parent_window));
-		parent_window = new_parent_window;
-	}
-}
-
-void
-nautilus_application_close_all_spatial_windows (void)
-{
-	GList *list_copy;
-	GList *l;
-	
-	list_copy = g_list_copy (nautilus_application_spatial_window_list);
-	/* First hide all window to get the feeling of quick response */
-	for (l = list_copy; l != NULL; l = l->next) {
-		NautilusWindow *window;
-		
-		window = NAUTILUS_WINDOW (l->data);
-		
-		if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			gtk_widget_hide (GTK_WIDGET (window));
-		}
-	}
-
-	for (l = list_copy; l != NULL; l = l->next) {
-		NautilusWindow *window;
-		
-		window = NAUTILUS_WINDOW (l->data);
-		
-		if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			nautilus_window_close (window);
-		}
+		nautilus_window_close (window);
 	}
 	g_list_free (list_copy);
 }
@@ -901,7 +595,6 @@ nautilus_window_delete_event_callback (GtkWidget *widget,
 
 static NautilusWindow *
 create_window (NautilusApplication *application,
-	       GType window_type,
 	       const char *startup_id,
 	       GdkScreen *screen)
 {
@@ -909,10 +602,10 @@ create_window (NautilusApplication *application,
 	
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	
-	window = NAUTILUS_WINDOW (gtk_widget_new (window_type,
-						  "app", application,
-						  "screen", screen,
-						  NULL));
+	window = g_object_new (NAUTILUS_TYPE_WINDOW,
+			       "app", application,
+			       "screen", screen,
+			       NULL);
 
 	if (startup_id) {
 		gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
@@ -933,75 +626,6 @@ create_window (NautilusApplication *application,
 	return window;
 }
 
-static void
-spatial_window_destroyed_callback (void *user_data, GObject *window)
-{
-	nautilus_application_spatial_window_list =
-		g_list_remove (nautilus_application_spatial_window_list, window);
-}
-
-NautilusWindow *
-nautilus_application_get_spatial_window (NautilusApplication *application,
-					 NautilusWindow      *requesting_window,
-					 const char          *startup_id,
-					 GFile               *location,
-					 GdkScreen           *screen,
-					 gboolean            *existing)
-{
-	NautilusWindow *window;
-
-	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
-
-	window = NAUTILUS_WINDOW
-		(nautilus_application_get_existing_spatial_window (location));
-
-	if (window != NULL) {
-		if (existing != NULL) {
-			*existing = TRUE;
-		}
-
-		DEBUG ("returning existing spatial window");
-
-		return window;
-	}
-
-	if (existing != NULL) {
-		*existing = FALSE;
-	}
-
-	window = create_window (application, NAUTILUS_TYPE_SPATIAL_WINDOW, startup_id, screen);
-	if (requesting_window) {
-		/* Center the window over the requesting window by default */
-		int orig_x, orig_y, orig_width, orig_height;
-		int new_x, new_y, new_width, new_height;
-		
-		gtk_window_get_position (GTK_WINDOW (requesting_window), 
-					 &orig_x, &orig_y);
-		gtk_window_get_size (GTK_WINDOW (requesting_window), 
-				     &orig_width, &orig_height);
-		gtk_window_get_default_size (GTK_WINDOW (window),
-					     &new_width, &new_height);
-		
-		new_x = orig_x + (orig_width - new_width) / 2;
-		new_y = orig_y + (orig_height - new_height) / 2;
-		
-		if (orig_width - new_width < 10) {
-			new_x += 10;
-			new_y += 10;
-		}
-
-		gtk_window_move (GTK_WINDOW (window), new_x, new_y);
-	}
-
-	nautilus_application_spatial_window_list = g_list_prepend (nautilus_application_spatial_window_list, window);
-	g_object_weak_ref (G_OBJECT (window), 
-			   spatial_window_destroyed_callback, NULL);
-
-	DEBUG ("Creating new spatial window");
-
-	return window;
-}
-
 static gboolean
 another_navigation_window_already_showing (NautilusApplication *application,
 					   NautilusWindow *the_window)
@@ -1010,8 +634,7 @@ another_navigation_window_already_showing (NautilusApplication *application,
 	
 	list = gtk_application_get_windows (GTK_APPLICATION (application));
 	for (item = list; item != NULL; item = item->next) {
-		if (item->data != the_window &&
-		    NAUTILUS_IS_NAVIGATION_WINDOW (item->data)) {
+		if (item->data != the_window) {
 			return TRUE;
 		}
 	}
@@ -1020,9 +643,9 @@ another_navigation_window_already_showing (NautilusApplication *application,
 }
 
 NautilusWindow *
-nautilus_application_create_navigation_window (NautilusApplication *application,
-					       const char          *startup_id,
-					       GdkScreen           *screen)
+nautilus_application_create_window (NautilusApplication *application,
+				    const char          *startup_id,
+				    GdkScreen           *screen)
 {
 	NautilusWindow *window;
 	char *geometry_string;
@@ -1030,7 +653,7 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 
-	window = create_window (application, NAUTILUS_TYPE_NAVIGATION_WINDOW, startup_id, screen);
+	window = create_window (application, startup_id, screen);
 
 	maximized = g_settings_get_boolean
 		(nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED);
@@ -1051,8 +674,8 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 		eel_gtk_window_set_initial_geometry_from_string 
 			(GTK_WINDOW (window), 
 			 geometry_string,
-			 NAUTILUS_NAVIGATION_WINDOW_MIN_WIDTH,
-			 NAUTILUS_NAVIGATION_WINDOW_MIN_HEIGHT,
+			 NAUTILUS_WINDOW_MIN_WIDTH,
+			 NAUTILUS_WINDOW_MIN_HEIGHT,
 			 another_navigation_window_already_showing (application, window));
 	}
 	g_free (geometry_string);
@@ -1094,10 +717,6 @@ mount_added_callback (GVolumeMonitor *monitor,
 	NautilusDirectory *directory;
 	GFile *root;
 	gchar *uri;
-
-	if (!application->priv->session_is_active) {
-		return;
-	}
 		
 	root = g_mount_get_root (mount);
 	uri = g_file_get_uri (root);
@@ -1119,9 +738,7 @@ get_first_navigation_slot (GList *slot_list)
 	GList *l;
 
 	for (l = slot_list; l != NULL; l = l->next) {
-		if (NAUTILUS_IS_NAVIGATION_WINDOW_SLOT (l->data)) {
-			return l->data;
-		}
+		return l->data;
 	}
 
 	return NULL;
@@ -1133,11 +750,7 @@ should_close_slot_with_mount (NautilusWindow *window,
 			      NautilusWindowSlot *slot,
 			      GMount *mount)
 {
-	if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-		return TRUE;
-	}
-	return nautilus_navigation_window_slot_should_close_with_mount (NAUTILUS_NAVIGATION_WINDOW_SLOT (slot),
-									mount);
+	return nautilus_window_slot_should_close_with_mount (slot, mount);
 }
 
 /* Called whenever a mount is unmounted. Check and see if there are
@@ -1215,7 +828,7 @@ mount_removed_callback (GVolumeMonitor *monitor,
 
 		if (should_close_slot_with_mount (window, slot, mount) &&
 		    slot != force_no_close_slot) {
-			nautilus_window_slot_close (slot);
+			nautilus_window_pane_slot_close (slot->pane, slot);
 		} else {
 			computer = g_file_new_for_uri ("computer:///");
 			nautilus_window_slot_go_to (slot, computer, FALSE);
@@ -1265,7 +878,6 @@ nautilus_application_finalize (GObject *object)
 
 	g_clear_object (&application->undo_manager);
 	g_clear_object (&application->priv->volume_monitor);
-	g_clear_object (&application->priv->ck_proxy);
 	g_clear_object (&application->priv->progress_handler);
 
 	nautilus_dbus_manager_stop ();
@@ -1292,7 +904,6 @@ nautilus_application_command_line (GApplication *app,
 	gboolean version = FALSE;
 	gboolean no_default_window = FALSE;
 	gboolean no_desktop = FALSE;
-	gboolean browser_window = FALSE;
 	gboolean kill_shell = FALSE;
 	gboolean autostart_mode = FALSE;
 	const gchar *autostart_id;
@@ -1311,8 +922,6 @@ nautilus_application_command_line (GApplication *app,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
 		{ "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &no_desktop,
 		  N_("Do not manage the desktop (ignore the preference set in the preferences dialog)."), NULL },
-		{ "browser", '\0', 0, G_OPTION_ARG_NONE, &browser_window, 
-		  N_("Open a browser window."), NULL },
 		{ "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
 		  N_("Quit Nautilus."), NULL },
 		{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining, NULL,  N_("[URI...]") },
@@ -1403,9 +1012,9 @@ nautilus_application_command_line (GApplication *app,
 		no_desktop = FALSE;
 	}
 
-	DEBUG ("Parsing command line, no_default_window %d browser %d, quit %d, "
+	DEBUG ("Parsing command line, no_default_window %d, quit %d, "
 	       "self checks %d, no_desktop %d",
-	       no_default_window, browser_window, kill_shell, perform_self_check, no_desktop);
+	       no_default_window, kill_shell, perform_self_check, no_desktop);
 
 	if (kill_shell) {
 		nautilus_application_quit (self);
@@ -1473,8 +1082,7 @@ nautilus_application_command_line (GApplication *app,
 			open_windows (self, NULL,
 				      uris,
 				      gdk_screen_get_default (),
-				      geometry,
-				      browser_window);
+				      geometry);
 		}
 	}
 
