@@ -33,6 +33,8 @@
 #include "nautilus-application.h"
 #include "nautilus-connect-server-dialog.h"
 #include "nautilus-file-management-properties.h"
+#include "nautilus-navigation-action.h"
+#include "nautilus-notebook.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-window-private.h"
@@ -41,7 +43,10 @@
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
+
 #include <eel/eel-gtk-extensions.h>
+#include <eel/eel-stock-dialogs.h>
+
 #include <libnautilus-extension/nautilus-menu-provider.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -63,163 +68,6 @@
 #define COMPUTER_URI         "computer:"
 #define BURN_CD_URI          "burn:"
 
-/* Struct that stores all the info necessary to activate a bookmark. */
-typedef struct {
-        NautilusBookmark *bookmark;
-        NautilusWindow *window;
-        guint changed_handler_id;
-	NautilusBookmarkFailedCallback failed_callback;
-} BookmarkHolder;
-
-static BookmarkHolder *
-bookmark_holder_new (NautilusBookmark *bookmark, 
-		     NautilusWindow *window,
-		     GCallback refresh_callback,
-		     NautilusBookmarkFailedCallback failed_callback)
-{
-	BookmarkHolder *new_bookmark_holder;
-
-	new_bookmark_holder = g_new (BookmarkHolder, 1);
-	new_bookmark_holder->window = window;
-	new_bookmark_holder->bookmark = bookmark;
-	new_bookmark_holder->failed_callback = failed_callback;
-	/* Ref the bookmark because it might be unreffed away while 
-	 * we're holding onto it (not an issue for window).
-	 */
-	g_object_ref (bookmark);
-	new_bookmark_holder->changed_handler_id = 
-		g_signal_connect_object (bookmark, "appearance_changed",
-					 refresh_callback,
-					 window, G_CONNECT_SWAPPED);
-
-	return new_bookmark_holder;
-}
-
-static void
-bookmark_holder_free (BookmarkHolder *bookmark_holder)
-{
-	g_signal_handler_disconnect (bookmark_holder->bookmark,
-				     bookmark_holder->changed_handler_id);
-	g_object_unref (bookmark_holder->bookmark);
-	g_free (bookmark_holder);
-}
-
-static void
-bookmark_holder_free_cover (gpointer callback_data, GClosure *closure)
-{
-	bookmark_holder_free (callback_data);
-}
-
-static gboolean
-should_open_in_new_tab (void)
-{
-	/* FIXME this is duplicated */
-	GdkEvent *event;
-
-	event = gtk_get_current_event ();
-
-	if (event == NULL) {
-		return FALSE;
-	}
-
-	if (event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) {
-		return event->button.button == 2;
-	}
-
-	gdk_event_free (event);
-
-	return FALSE;
-}
-
-static void
-activate_bookmark_in_menu_item (GtkAction *action, gpointer user_data)
-{
-	NautilusWindowSlot *slot;
-        BookmarkHolder *holder;
-        GFile *location;
-
-        holder = (BookmarkHolder *)user_data;
-
-	if (nautilus_bookmark_uri_known_not_to_exist (holder->bookmark)) {
-		holder->failed_callback (holder->window, holder->bookmark);
-	} else {
-	        location = nautilus_bookmark_get_location (holder->bookmark);
-		slot = nautilus_window_get_active_slot (holder->window);
-	        nautilus_window_slot_go_to (slot, 
-					    location, 
-					    should_open_in_new_tab ());
-	        g_object_unref (location);
-        }
-}
-
-void
-nautilus_menus_append_bookmark_to_menu (NautilusWindow *window, 
-					NautilusBookmark *bookmark, 
-					const char *parent_path,
-					const char *parent_id,
-					guint index_in_parent,
-					GtkActionGroup *action_group,
-					guint merge_id,
-					GCallback refresh_callback,
-					NautilusBookmarkFailedCallback failed_callback)
-{
-	BookmarkHolder *bookmark_holder;
-	char action_name[128];
-	char *name;
-	char *path;
-	GdkPixbuf *pixbuf;
-	GtkAction *action;
-	GtkWidget *menuitem;
-
-	g_assert (NAUTILUS_IS_WINDOW (window));
-	g_assert (NAUTILUS_IS_BOOKMARK (bookmark));
-
-	bookmark_holder = bookmark_holder_new (bookmark, window, refresh_callback, failed_callback);
-	name = nautilus_bookmark_get_name (bookmark);
-
-	/* Create menu item with pixbuf */
-	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, GTK_ICON_SIZE_MENU);
-
-	g_snprintf (action_name, sizeof (action_name), "%s%d", parent_id, index_in_parent);
-
-	action = gtk_action_new (action_name,
-				 name,
-				 _("Go to the location specified by this bookmark"),
-				 NULL);
-	
-	g_object_set_data_full (G_OBJECT (action), "menu-icon",
-				g_object_ref (pixbuf),
-				g_object_unref);
-
-	g_signal_connect_data (action, "activate",
-			       G_CALLBACK (activate_bookmark_in_menu_item),
-			       bookmark_holder, 
-			       bookmark_holder_free_cover, 0);
-
-	gtk_action_group_add_action (action_group,
-				     GTK_ACTION (action));
-
-	g_object_unref (action);
-
-	gtk_ui_manager_add_ui (window->details->ui_manager,
-			       merge_id,
-			       parent_path,
-			       action_name,
-			       action_name,
-			       GTK_UI_MANAGER_MENUITEM,
-			       FALSE);
-
-	path = g_strdup_printf ("%s/%s", parent_path, action_name);
-	menuitem = gtk_ui_manager_get_widget (window->details->ui_manager,
-					      path);
-	gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menuitem),
-						   TRUE);
-
-	g_object_unref (pixbuf);
-	g_free (path);
-	g_free (name);
-}
-
 static void
 action_close_window_slot_callback (GtkAction *action,
 				   gpointer user_data)
@@ -230,7 +78,7 @@ action_close_window_slot_callback (GtkAction *action,
 	window = NAUTILUS_WINDOW (user_data);
 	slot = nautilus_window_get_active_slot (window);
 
-	nautilus_window_slot_close (slot);
+	nautilus_window_pane_slot_close (slot->pane, slot);
 }
 
 static void
@@ -262,8 +110,12 @@ static void
 action_undo_callback (GtkAction *action, 
 		      gpointer user_data) 
 {
-	nautilus_undo_manager_undo
-		(NAUTILUS_WINDOW (user_data)->application->undo_manager);
+	NautilusApplication *app;
+
+	app = nautilus_application_dup_singleton ();
+	nautilus_undo_manager_undo (app->undo_manager);
+
+	g_object_unref (app);
 }
 
 static void
@@ -277,7 +129,7 @@ action_home_callback (GtkAction *action,
 	slot = nautilus_window_get_active_slot (window);
 
 	nautilus_window_slot_go_home (slot, 
-				      should_open_in_new_tab ());
+				      nautilus_event_should_open_in_new_tab ());
 }
 
 static void
@@ -294,7 +146,7 @@ action_go_to_computer_callback (GtkAction *action,
 	computer = g_file_new_for_uri (COMPUTER_URI);
 	nautilus_window_slot_go_to (slot,
 				    computer,
-				    should_open_in_new_tab ());
+				    nautilus_event_should_open_in_new_tab ());
 	g_object_unref (computer);
 }
 
@@ -312,7 +164,7 @@ action_go_to_network_callback (GtkAction *action,
 	network = g_file_new_for_uri (NETWORK_URI);
 	nautilus_window_slot_go_to (slot,
 				    network,
-				    should_open_in_new_tab ());
+				    nautilus_event_should_open_in_new_tab ());
 	g_object_unref (network);
 }
 
@@ -333,7 +185,7 @@ action_go_to_templates_callback (GtkAction *action,
 	g_free (path);
 	nautilus_window_slot_go_to (slot,
 				    location,
-				    should_open_in_new_tab ());
+				    nautilus_event_should_open_in_new_tab ());
 	g_object_unref (location);
 }
 
@@ -351,7 +203,7 @@ action_go_to_trash_callback (GtkAction *action,
 	trash = g_file_new_for_uri ("trash:///");
 	nautilus_window_slot_go_to (slot,
 				    trash,
-				    should_open_in_new_tab ());
+				    nautilus_event_should_open_in_new_tab ());
 	g_object_unref (trash);
 }
 
@@ -359,28 +211,44 @@ static void
 action_reload_callback (GtkAction *action, 
 			gpointer user_data) 
 {
-	nautilus_window_reload (NAUTILUS_WINDOW (user_data));
+	NautilusWindowSlot *slot;
+
+	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
+	nautilus_window_slot_reload (slot);
+}
+
+static NautilusView *
+get_current_view (NautilusWindow *window)
+{
+	NautilusWindowSlot *slot;
+	NautilusView *view;
+
+	slot = nautilus_window_get_active_slot (window);
+	view = nautilus_window_slot_get_current_view (slot);
+
+	return view;
 }
 
 static void
 action_zoom_in_callback (GtkAction *action, 
 			 gpointer user_data) 
 {
-	nautilus_window_zoom_in (NAUTILUS_WINDOW (user_data));
+
+	nautilus_view_bump_zoom_level (get_current_view (user_data), 1);
 }
 
 static void
 action_zoom_out_callback (GtkAction *action, 
 			  gpointer user_data) 
 {
-	nautilus_window_zoom_out (NAUTILUS_WINDOW (user_data));
+	nautilus_view_bump_zoom_level (get_current_view (user_data), -1);
 }
 
 static void
 action_zoom_normal_callback (GtkAction *action, 
 			     gpointer user_data) 
 {
-	nautilus_window_zoom_to_default (NAUTILUS_WINDOW (user_data));
+	nautilus_view_restore_default_zoom_level (get_current_view (user_data));
 }
 
 static void
@@ -398,7 +266,7 @@ action_show_hidden_files_callback (GtkAction *action,
 		mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE;
 	}
 
-	nautilus_window_info_set_hidden_files_mode (window, mode);
+	nautilus_window_set_hidden_files_mode (window, mode);
 }
 
 static void
@@ -420,7 +288,7 @@ show_hidden_files_preference_callback (gpointer callback_data)
 		g_signal_handlers_unblock_by_func (action, action_show_hidden_files_callback, window);
 
 		/* inform views */
-		nautilus_window_info_set_hidden_files_mode (window, NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT);
+		nautilus_window_set_hidden_files_mode (window, NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT);
 
 	}
 }
@@ -532,7 +400,11 @@ static void
 action_up_callback (GtkAction *action, 
 		     gpointer user_data) 
 {
-	nautilus_window_go_up (NAUTILUS_WINDOW (user_data), FALSE, should_open_in_new_tab ());
+	NautilusWindow *window = user_data;
+	NautilusWindowSlot *slot;
+
+	slot = nautilus_window_get_active_slot (window);
+	nautilus_window_slot_go_up (slot, FALSE, nautilus_event_should_open_in_new_tab ());
 }
 
 static void
@@ -597,127 +469,18 @@ menu_item_deselect_cb (GtkMenuItem *proxy,
 			   window->details->help_message_cid);
 }
 
-static GtkWidget *
-get_event_widget (GtkWidget *proxy)
-{
-	GtkWidget *widget;
-
-	/**
-	 * Finding the interesting widget requires internal knowledge of
-	 * the widgets in question. This can't be helped, but by keeping
-	 * the sneaky code in one place, it can easily be updated.
-	 */
-	if (GTK_IS_MENU_ITEM (proxy)) {
-		/* Menu items already forward middle clicks */
-		widget = NULL;
-	} else if (GTK_IS_MENU_TOOL_BUTTON (proxy)) {
-		widget = eel_gtk_menu_tool_button_get_button (GTK_MENU_TOOL_BUTTON (proxy));
-	} else if (GTK_IS_TOOL_BUTTON (proxy)) {
-		/* The tool button's button is the direct child */
-		widget = gtk_bin_get_child (GTK_BIN (proxy));
-	} else if (GTK_IS_BUTTON (proxy)) {
-		widget = proxy;
-	} else {
-		/* Don't touch anything we don't know about */
-		widget = NULL;
-	}
-
-	return widget;
-}
-
-static gboolean
-proxy_button_press_event_cb (GtkButton *button,
-			     GdkEventButton *event,
-			     gpointer user_data)
-{
-	if (event->button == 2) {
-		g_signal_emit_by_name (button, "pressed", 0);
-	}
-
-	return FALSE;
-}
-
-static gboolean
-proxy_button_release_event_cb (GtkButton *button,
-			       GdkEventButton *event,
-			       gpointer user_data)
-{
-	if (event->button == 2) {
-		g_signal_emit_by_name (button, "released", 0);
-	}
-
-	return FALSE;
-}
-
 static void
 disconnect_proxy_cb (GtkUIManager *manager,
 		     GtkAction *action,
 		     GtkWidget *proxy,
 		     NautilusWindow *window)
 {
-	GtkWidget *widget;
-
 	if (GTK_IS_MENU_ITEM (proxy)) {
 		g_signal_handlers_disconnect_by_func
 			(proxy, G_CALLBACK (menu_item_select_cb), window);
 		g_signal_handlers_disconnect_by_func
 			(proxy, G_CALLBACK (menu_item_deselect_cb), window);
 	}
-
-	widget = get_event_widget (proxy);
-	if (widget) {
-		g_signal_handlers_disconnect_by_func (widget,
-						      G_CALLBACK (proxy_button_press_event_cb),
-						      action);
-		g_signal_handlers_disconnect_by_func (widget,
-						      G_CALLBACK (proxy_button_release_event_cb),
-						      action);
-	}
-
-}
-
-static void
-connect_proxy_cb (GtkUIManager *manager,
-		  GtkAction *action,
-		  GtkWidget *proxy,
-		  NautilusWindow *window)
-{
-	GdkPixbuf *icon;
-	GtkWidget *widget;
-	
-	if (GTK_IS_MENU_ITEM (proxy)) {
-		g_signal_connect (proxy, "select",
-				  G_CALLBACK (menu_item_select_cb), window);
-		g_signal_connect (proxy, "deselect",
-				  G_CALLBACK (menu_item_deselect_cb), window);
-
-
-		/* This is a way to easily get pixbufs into the menu items */
-		icon = g_object_get_data (G_OBJECT (action), "menu-icon");
-		if (icon != NULL) {
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy),
-						       gtk_image_new_from_pixbuf (icon));
-		}
-	}
-	if (GTK_IS_TOOL_BUTTON (proxy)) {
-		icon = g_object_get_data (G_OBJECT (action), "toolbar-icon");
-		if (icon != NULL) {
-			widget = gtk_image_new_from_pixbuf (icon);
-			gtk_widget_show (widget);
-			gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (proxy),
-							 widget);
-		}
-	}
-	
-	widget = get_event_widget (proxy);
-	if (widget) {
-		g_signal_connect (widget, "button-press-event",
-				  G_CALLBACK (proxy_button_press_event_cb),
-				  action);
-		g_signal_connect (widget, "button-release-event",
-				  G_CALLBACK (proxy_button_release_event_cb),
-				  action);
-	}
 }
 
 static void
@@ -753,6 +516,424 @@ nautilus_window_initialize_trash_icon_monitor (NautilusWindow *window)
 			  G_CALLBACK (trash_state_changed_cb), window);
 }
 
+#define MENU_PATH_HISTORY_PLACEHOLDER			"/MenuBar/Other Menus/Go/History Placeholder"
+
+#define RESPONSE_FORGET		1000
+#define MENU_ITEM_MAX_WIDTH_CHARS 32
+
+enum {
+	SIDEBAR_PLACES,
+	SIDEBAR_TREE
+};
+
+static void
+action_close_all_windows_callback (GtkAction *action, 
+				   gpointer user_data)
+{
+	NautilusApplication *app;
+
+	app = nautilus_application_dup_singleton ();
+	nautilus_application_close_all_windows (app);
+
+	g_object_unref (app);
+}
+
+static void
+action_back_callback (GtkAction *action, 
+		      gpointer user_data) 
+{
+	nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data), 
+					 TRUE, 0, nautilus_event_should_open_in_new_tab ());
+}
+
+static void
+action_forward_callback (GtkAction *action, 
+			 gpointer user_data) 
+{
+	nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data), 
+					 FALSE, 0, nautilus_event_should_open_in_new_tab ());
+}
+
+static void
+forget_history_if_yes (GtkDialog *dialog, int response, gpointer callback_data)
+{
+	if (response == RESPONSE_FORGET) {
+		nautilus_forget_history ();
+	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+forget_history_if_confirmed (NautilusWindow *window)
+{
+	GtkDialog *dialog;
+
+	dialog = eel_create_question_dialog (_("Are you sure you want to clear the list "
+					       "of locations you have visited?"),
+					     NULL,
+					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     GTK_STOCK_CLEAR, RESPONSE_FORGET,
+					     GTK_WINDOW (window));
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+	
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (forget_history_if_yes), NULL);
+
+	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
+}
+
+static void
+action_clear_history_callback (GtkAction *action, 
+			       gpointer user_data) 
+{
+	forget_history_if_confirmed (NAUTILUS_WINDOW (user_data));
+}
+
+static void
+action_split_view_switch_next_pane_callback(GtkAction *action,
+					    gpointer user_data)
+{
+	nautilus_window_pane_grab_focus (nautilus_window_get_next_pane (NAUTILUS_WINDOW (user_data)));
+}
+
+static void
+action_split_view_same_location_callback (GtkAction *action,
+					  gpointer user_data)
+{
+	NautilusWindow *window;
+	NautilusWindowPane *next_pane;
+	GFile *location;
+
+	window = NAUTILUS_WINDOW (user_data);
+	next_pane = nautilus_window_get_next_pane (window);
+
+	if (!next_pane) {
+		return;
+	}
+	location = nautilus_window_slot_get_location (next_pane->active_slot);
+	if (location) {
+		nautilus_window_slot_go_to (window->details->active_pane->active_slot, location, FALSE);
+		g_object_unref (location);
+	}
+}
+
+static void
+action_show_hide_sidebar_callback (GtkAction *action, 
+				   gpointer user_data)
+{
+	NautilusWindow *window;
+
+	window = NAUTILUS_WINDOW (user_data);
+
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
+		nautilus_window_show_sidebar (window);
+	} else {
+		nautilus_window_hide_sidebar (window);
+	}
+}
+
+static void
+action_split_view_callback (GtkAction *action,
+			    gpointer user_data)
+{
+	NautilusWindow *window;
+	gboolean is_active;
+
+	window = NAUTILUS_WINDOW (user_data);
+
+	is_active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	if (is_active != nautilus_window_split_view_showing (window)) {
+		NautilusWindow *nautilus_window;
+
+		if (is_active) {
+			nautilus_window_split_view_on (window);
+		} else {
+			nautilus_window_split_view_off (window);
+		}
+		nautilus_window = NAUTILUS_WINDOW (window);
+		if (nautilus_window->details->active_pane && nautilus_window->details->active_pane->active_slot) {
+			nautilus_view_update_menus (nautilus_window->details->active_pane->active_slot->content_view);
+		}
+	}
+}
+
+
+/* TODO: bind all of this with g_settings_bind and GBinding */
+static guint
+sidebar_id_to_value (const gchar *sidebar_id)
+{
+	guint retval = SIDEBAR_PLACES;
+
+	if (g_strcmp0 (sidebar_id, NAUTILUS_WINDOW_SIDEBAR_TREE) == 0)
+		retval = SIDEBAR_TREE;
+
+	return retval;
+}
+
+void
+nautilus_window_update_show_hide_menu_items (NautilusWindow *window) 
+{
+	GtkAction *action;
+	guint current_value;
+
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	action = gtk_action_group_get_action (window->details->main_action_group,
+					      NAUTILUS_ACTION_SHOW_HIDE_EXTRA_PANE);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      nautilus_window_split_view_showing (window));
+
+	action = gtk_action_group_get_action (window->details->main_action_group,
+					      "Sidebar Places");
+	current_value = sidebar_id_to_value (window->details->sidebar_id);
+	gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), current_value);
+}
+
+static void
+action_add_bookmark_callback (GtkAction *action,
+			      gpointer user_data)
+{
+        nautilus_window_add_bookmark_for_current_location (NAUTILUS_WINDOW (user_data));
+}
+
+static void
+action_edit_bookmarks_callback (GtkAction *action, 
+				gpointer user_data)
+{
+        nautilus_window_edit_bookmarks (NAUTILUS_WINDOW (user_data));
+}
+
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+                  GtkAction *action,
+                  GtkWidget *proxy,
+                  NautilusWindow *window)
+{
+	GtkLabel *label;
+
+	if (!GTK_IS_MENU_ITEM (proxy))
+		return;
+
+	label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (proxy)));
+
+	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (label, MENU_ITEM_MAX_WIDTH_CHARS);
+
+	g_signal_connect (proxy, "select",
+			  G_CALLBACK (menu_item_select_cb), window);
+	g_signal_connect (proxy, "deselect",
+			  G_CALLBACK (menu_item_deselect_cb), window);
+}
+
+static const char* icon_entries[] = {
+	"/MenuBar/Other Menus/Go/Home",
+	"/MenuBar/Other Menus/Go/Computer",
+	"/MenuBar/Other Menus/Go/Go to Templates",
+	"/MenuBar/Other Menus/Go/Go to Trash",
+	"/MenuBar/Other Menus/Go/Go to Network",
+	"/MenuBar/Other Menus/Go/Go to Location"
+};
+
+/**
+ * refresh_go_menu:
+ * 
+ * Refresh list of bookmarks at end of Go menu to match centralized history list.
+ * @window: The NautilusWindow whose Go menu will be refreshed.
+ **/
+static void
+nautilus_window_initialize_go_menu (NautilusWindow *window)
+{
+	GtkUIManager *ui_manager;
+	GtkWidget *menuitem;
+	int i;
+
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	ui_manager = nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window));
+
+	for (i = 0; i < G_N_ELEMENTS (icon_entries); i++) {
+		menuitem = gtk_ui_manager_get_widget (
+				ui_manager,
+				icon_entries[i]);
+
+		gtk_image_menu_item_set_always_show_image (
+				GTK_IMAGE_MENU_ITEM (menuitem), TRUE);
+	}
+}
+
+void
+nautilus_window_update_split_view_actions_sensitivity (NautilusWindow *window)
+{
+	NautilusWindow *win;
+	GtkActionGroup *action_group;
+	GtkAction *action;
+	gboolean have_multiple_panes;
+	gboolean next_pane_is_in_same_location;
+	GFile *active_pane_location;
+	GFile *next_pane_location;
+	NautilusWindowPane *next_pane;
+
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	action_group = window->details->main_action_group;
+	win = NAUTILUS_WINDOW (window);
+
+	/* collect information */
+	have_multiple_panes = (win->details->panes && win->details->panes->next);
+	if (win->details->active_pane->active_slot) {
+		active_pane_location = nautilus_window_slot_get_location (win->details->active_pane->active_slot);
+	}
+	else {
+		active_pane_location = NULL;
+	}
+	next_pane = nautilus_window_get_next_pane (win);
+	if (next_pane && next_pane->active_slot) {
+		next_pane_location = nautilus_window_slot_get_location (next_pane->active_slot);
+		next_pane_is_in_same_location = (active_pane_location && next_pane_location &&
+						 g_file_equal (active_pane_location, next_pane_location));
+	}
+	else {
+		next_pane_location = NULL;
+		next_pane_is_in_same_location = FALSE;
+	}
+
+	/* switch to next pane */
+	action = gtk_action_group_get_action (action_group, "SplitViewNextPane");
+	gtk_action_set_sensitive (action, have_multiple_panes);
+
+	/* same location */
+	action = gtk_action_group_get_action (action_group, "SplitViewSameLocation");
+	gtk_action_set_sensitive (action, have_multiple_panes && !next_pane_is_in_same_location);
+
+	/* clean up */
+	if (active_pane_location) {
+		g_object_unref (active_pane_location);
+	}
+	if (next_pane_location) {
+		g_object_unref (next_pane_location);
+	}
+}
+
+static void
+action_new_window_callback (GtkAction *action,
+			    gpointer user_data)
+{
+	NautilusApplication *application;
+	NautilusWindow *current_window, *new_window;
+
+	current_window = NAUTILUS_WINDOW (user_data);
+	application = nautilus_application_dup_singleton ();
+
+	new_window = nautilus_application_create_window (
+				application,
+				NULL,
+				gtk_window_get_screen (GTK_WINDOW (current_window)));
+	nautilus_window_slot_go_home (nautilus_window_get_active_slot (new_window), FALSE);
+
+	g_object_unref (application);
+}
+
+static void
+action_new_tab_callback (GtkAction *action,
+			 gpointer user_data)
+{
+	NautilusWindow *window;
+
+	window = NAUTILUS_WINDOW (user_data);
+	nautilus_window_new_tab (window);
+}
+
+static void
+action_go_to_location_callback (GtkAction *action,
+				gpointer user_data)
+{
+	NautilusWindow *window;
+
+	window = NAUTILUS_WINDOW (user_data);
+
+	nautilus_window_prompt_for_location (window, NULL);
+}
+
+static void
+action_tabs_previous_callback (GtkAction *action,
+			       gpointer user_data)
+{
+	NautilusWindowPane *pane;
+
+	pane = NAUTILUS_WINDOW (user_data)->details->active_pane;
+	nautilus_notebook_set_current_page_relative (NAUTILUS_NOTEBOOK (pane->notebook), -1);
+}
+
+static void
+action_tabs_next_callback (GtkAction *action,
+			   gpointer user_data)
+{
+	NautilusWindowPane *pane;
+
+	pane = NAUTILUS_WINDOW (user_data)->details->active_pane;
+	nautilus_notebook_set_current_page_relative (NAUTILUS_NOTEBOOK (pane->notebook), 1);
+}
+
+static void
+action_tabs_move_left_callback (GtkAction *action,
+				gpointer user_data)
+{
+	NautilusWindowPane *pane;
+
+	pane = NAUTILUS_WINDOW (user_data)->details->active_pane;
+	nautilus_notebook_reorder_current_child_relative (NAUTILUS_NOTEBOOK (pane->notebook), -1);
+}
+
+static void
+action_tabs_move_right_callback (GtkAction *action,
+				 gpointer user_data)
+{
+	NautilusWindowPane *pane;
+
+	pane = NAUTILUS_WINDOW (user_data)->details->active_pane;
+	nautilus_notebook_reorder_current_child_relative (NAUTILUS_NOTEBOOK (pane->notebook), 1);
+}
+
+static void
+action_tab_change_action_activate_callback (GtkAction *action, gpointer user_data)
+{
+	NautilusWindow *window;
+
+	window = NAUTILUS_WINDOW (user_data);
+	if (window && window->details->active_pane) {
+		GtkNotebook *notebook;
+		notebook = GTK_NOTEBOOK (window->details->active_pane->notebook);
+		if (notebook) {
+			int num;
+			num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), "num"));
+			if (num < gtk_notebook_get_n_pages (notebook)) {
+				gtk_notebook_set_current_page (notebook, num);
+			}
+		}
+	}
+}
+
+static void
+sidebar_radio_entry_changed_cb (GtkAction *action,
+				GtkRadioAction *current,
+				gpointer user_data)
+{
+	gint current_value;
+
+	current_value = gtk_radio_action_get_current_value (current);
+
+	if (current_value == SIDEBAR_PLACES) {
+		g_settings_set_string (nautilus_window_state,
+				       NAUTILUS_WINDOW_STATE_SIDE_PANE_VIEW,
+				       NAUTILUS_WINDOW_SIDEBAR_PLACES);
+	} else if (current_value == SIDEBAR_TREE) {
+		g_settings_set_string (nautilus_window_state,
+				       NAUTILUS_WINDOW_STATE_SIDE_PANE_VIEW,
+				       NAUTILUS_WINDOW_SIDEBAR_TREE);
+	}
+}
+
 static const GtkActionEntry main_entries[] = {
   /* name, stock id, label */  { "File", NULL, N_("_File") },
   /* name, stock id, label */  { "Edit", NULL, N_("_Edit") },
@@ -839,6 +1020,55 @@ static const GtkActionEntry main_entries[] = {
   /* label, accelerator */       N_("_Trash"), NULL,
   /* tooltip */                  N_("Open your personal trash folder"),
                                  G_CALLBACK (action_go_to_trash_callback) },
+  /* name, stock id, label */  { "Go", NULL, N_("_Go") },
+  /* name, stock id, label */  { "Bookmarks", NULL, N_("_Bookmarks") },
+  /* name, stock id, label */  { "Tabs", NULL, N_("_Tabs") },
+  /* name, stock id, label */  { "New Window", "window-new", N_("New _Window"),
+                                 "<control>N", N_("Open another Nautilus window for the displayed location"),
+                                 G_CALLBACK (action_new_window_callback) },
+  /* name, stock id, label */  { "New Tab", "tab-new", N_("New _Tab"),
+                                 "<control>T", N_("Open another tab for the displayed location"),
+                                 G_CALLBACK (action_new_tab_callback) },
+  /* name, stock id, label */  { "Close All Windows", NULL, N_("Close _All Windows"),
+                                 "<control>Q", N_("Close all Navigation windows"),
+                                 G_CALLBACK (action_close_all_windows_callback) },
+  /* name, stock id, label */  { NAUTILUS_ACTION_BACK, GTK_STOCK_GO_BACK, N_("_Back"),
+				 "<alt>Left", N_("Go to the previous visited location"),
+				 G_CALLBACK (action_back_callback) },
+  /* name, stock id, label */  { NAUTILUS_ACTION_FORWARD, GTK_STOCK_GO_FORWARD, N_("_Forward"),
+				 "<alt>Right", N_("Go to the next visited location"),
+				 G_CALLBACK (action_forward_callback) },
+  /* name, stock id, label */  { "Go to Location", NULL, N_("_Location..."),
+                                 "<control>L", N_("Specify a location to open"),
+                                 G_CALLBACK (action_go_to_location_callback) },
+  /* name, stock id, label */  { "Clear History", NULL, N_("Clea_r History"),
+                                 NULL, N_("Clear contents of Go menu and Back/Forward lists"),
+                                 G_CALLBACK (action_clear_history_callback) },
+  /* name, stock id, label */  { "SplitViewNextPane", NULL, N_("S_witch to Other Pane"),
+				 "F6", N_("Move focus to the other pane in a split view window"),
+				 G_CALLBACK (action_split_view_switch_next_pane_callback) },
+  /* name, stock id, label */  { "SplitViewSameLocation", NULL, N_("Sa_me Location as Other Pane"),
+				 NULL, N_("Go to the same location as in the extra pane"),
+				 G_CALLBACK (action_split_view_same_location_callback) },
+  /* name, stock id, label */  { "Add Bookmark", GTK_STOCK_ADD, N_("_Add Bookmark"),
+                                 "<control>d", N_("Add a bookmark for the current location to this menu"),
+                                 G_CALLBACK (action_add_bookmark_callback) },
+  /* name, stock id, label */  { "Edit Bookmarks", NULL, N_("_Edit Bookmarks..."),
+                                 "<control>b", N_("Display a window that allows editing the bookmarks in this menu"),
+                                 G_CALLBACK (action_edit_bookmarks_callback) },
+  { "TabsPrevious", NULL, N_("_Previous Tab"), "<control>Page_Up",
+    N_("Activate previous tab"),
+    G_CALLBACK (action_tabs_previous_callback) },
+  { "TabsNext", NULL, N_("_Next Tab"), "<control>Page_Down",
+    N_("Activate next tab"),
+    G_CALLBACK (action_tabs_next_callback) },
+  { "TabsMoveLeft", NULL, N_("Move Tab _Left"), "<shift><control>Page_Up",
+    N_("Move current tab to left"),
+    G_CALLBACK (action_tabs_move_left_callback) },
+  { "TabsMoveRight", NULL, N_("Move Tab _Right"), "<shift><control>Page_Down",
+    N_("Move current tab to right"),
+    G_CALLBACK (action_tabs_move_right_callback) },
+  { "Sidebar List", NULL, N_("Sidebar") }
 };
 
 static const GtkToggleActionEntry main_toggle_entries[] = {
@@ -847,8 +1077,152 @@ static const GtkToggleActionEntry main_toggle_entries[] = {
   /* tooltip */                  N_("Toggle the display of hidden files in the current window"),
                                  G_CALLBACK (action_show_hidden_files_callback),
                                  TRUE },
+  /* name, stock id */     { "Show Hide Toolbar", NULL,
+  /* label, accelerator */   N_("_Main Toolbar"), NULL,
+  /* tooltip */              N_("Change the visibility of this window's main toolbar"),
+			     NULL,
+  /* is_active */            TRUE }, 
+  /* name, stock id */     { "Show Hide Sidebar", NULL,
+  /* label, accelerator */   N_("_Show Sidebar"), "F9",
+  /* tooltip */              N_("Change the visibility of this window's side pane"),
+                             G_CALLBACK (action_show_hide_sidebar_callback),
+  /* is_active */            TRUE }, 
+  /* name, stock id */     { "Show Hide Statusbar", NULL,
+  /* label, accelerator */   N_("St_atusbar"), NULL,
+  /* tooltip */              N_("Change the visibility of this window's statusbar"),
+                             NULL,
+  /* is_active */            TRUE },
+  /* name, stock id */     { "Search", "edit-find-symbolic",
+  /* label, accelerator */   N_("_Search for Files..."), "<control>f",
+  /* tooltip */              N_("Search documents and folders by name"),
+			     NULL,
+  /* is_active */            FALSE },
+  /* name, stock id */     { NAUTILUS_ACTION_SHOW_HIDE_EXTRA_PANE, NULL,
+  /* label, accelerator */   N_("E_xtra Pane"), "F3",
+  /* tooltip */              N_("Open an extra folder view side-by-side"),
+                             G_CALLBACK (action_split_view_callback),
+  /* is_active */            FALSE },
 };
 
+static const GtkRadioActionEntry main_radio_entries[] = {
+	{ "Sidebar Places", NULL,
+	  N_("Places"), NULL, N_("Select Places as the default sidebar"),
+	  SIDEBAR_PLACES },
+	{ "Sidebar Tree", NULL,
+	  N_("Tree"), NULL, N_("Select Tree as the default sidebar"),
+	  SIDEBAR_TREE }
+};
+
+GtkActionGroup *
+nautilus_window_create_toolbar_action_group (NautilusWindow *window)
+{
+	GtkActionGroup *action_group;
+	GtkAction *action;
+
+	action_group = gtk_action_group_new ("ToolbarActions");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+
+	action = g_object_new (NAUTILUS_TYPE_NAVIGATION_ACTION,
+			       "name", NAUTILUS_ACTION_BACK,
+			       "label", _("_Back"),
+			       "stock_id", GTK_STOCK_GO_BACK,
+			       "tooltip", _("Go to the previous visited location"),
+			       "arrow-tooltip", _("Back history"),
+			       "window", window,
+			       "direction", NAUTILUS_NAVIGATION_DIRECTION_BACK,
+			       NULL);
+	g_signal_connect (action, "activate",
+			  G_CALLBACK (action_back_callback), window);
+	gtk_action_group_add_action (action_group, action);
+
+	g_object_unref (action);
+
+	action = g_object_new (NAUTILUS_TYPE_NAVIGATION_ACTION,
+			       "name", NAUTILUS_ACTION_FORWARD,
+			       "label", _("_Forward"),
+			       "stock_id", GTK_STOCK_GO_FORWARD,
+			       "tooltip", _("Go to the next visited location"),
+			       "arrow-tooltip", _("Forward history"),
+			       "window", window,
+			       "direction", NAUTILUS_NAVIGATION_DIRECTION_FORWARD,
+			       NULL);
+	g_signal_connect (action, "activate",
+			  G_CALLBACK (action_forward_callback), window);
+	gtk_action_group_add_action (action_group, action);
+
+	g_object_unref (action);
+
+	action = GTK_ACTION
+		(gtk_toggle_action_new (NAUTILUS_ACTION_SEARCH,
+					_("Search"),
+					_("Search documents and folders by name"),
+					NULL));
+	gtk_action_group_add_action (action_group, action);
+	gtk_action_set_icon_name (GTK_ACTION (action), "edit-find-symbolic");
+	gtk_action_set_is_important (GTK_ACTION (action), TRUE);
+
+	g_object_unref (action);
+
+	nautilus_navigation_state_add_group (window->details->nav_state,
+					     action_group);
+
+	return action_group;
+}
+
+static void
+window_menus_set_bindings (NautilusWindow *window)
+{
+	GtkAction *action;
+
+	action = gtk_action_group_get_action (window->details->main_action_group,
+					      NAUTILUS_ACTION_SHOW_HIDE_TOOLBAR);
+
+	g_settings_bind (nautilus_window_state,
+			 NAUTILUS_WINDOW_STATE_START_WITH_TOOLBAR,
+			 action,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	action = gtk_action_group_get_action (window->details->main_action_group,
+					      NAUTILUS_ACTION_SHOW_HIDE_STATUSBAR);
+
+	g_settings_bind (nautilus_window_state,
+			 NAUTILUS_WINDOW_STATE_START_WITH_STATUS_BAR,
+			 action,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	action = gtk_action_group_get_action (window->details->main_action_group,
+					      NAUTILUS_ACTION_SHOW_HIDE_SIDEBAR);	
+
+	g_settings_bind (nautilus_window_state,
+			 NAUTILUS_WINDOW_STATE_START_WITH_SIDEBAR,
+			 action,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+}
+
+void 
+nautilus_window_initialize_actions (NautilusWindow *window)
+{
+	GtkActionGroup *action_group;
+	const gchar *nav_state_actions[] = {
+		NAUTILUS_ACTION_BACK, NAUTILUS_ACTION_FORWARD,
+		NAUTILUS_ACTION_SEARCH, NULL
+	};
+
+	action_group = window->details->main_action_group;
+	window->details->nav_state = nautilus_navigation_state_new (action_group,
+								    nav_state_actions);
+
+	window_menus_set_bindings (window);
+	nautilus_window_update_show_hide_menu_items (window);
+
+	g_signal_connect (window, "loading_uri",
+			  G_CALLBACK (nautilus_window_update_split_view_actions_sensitivity),
+			  NULL);
+}
+
 /**
  * nautilus_window_initialize_menus
  * 
@@ -862,7 +1236,12 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 	GtkUIManager *ui_manager;
 	GtkAction *action;
 	const char *ui;
-	
+	gint i;
+
+	window->details->ui_manager = gtk_ui_manager_new ();
+	ui_manager = window->details->ui_manager;
+
+	/* shell actions */
 	action_group = gtk_action_group_new ("ShellActions");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
 	window->details->main_action_group = action_group;
@@ -872,6 +1251,10 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 	gtk_action_group_add_toggle_actions (action_group, 
 					     main_toggle_entries, G_N_ELEMENTS (main_toggle_entries),
 					     window);
+	gtk_action_group_add_radio_actions (action_group,
+					    main_radio_entries, G_N_ELEMENTS (main_radio_entries),
+					    0, G_CALLBACK (sidebar_radio_entry_changed_cb),
+					    window);
 
 	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_UP);
 	g_object_set (action, "short_label", _("_Up"), NULL);
@@ -890,8 +1273,32 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 				  G_CALLBACK(show_hidden_files_preference_callback),
 				  window);
 
-	window->details->ui_manager = gtk_ui_manager_new ();
-	ui_manager = window->details->ui_manager;
+	/* Alt+N for the first 10 tabs */
+	for (i = 0; i < 10; ++i) {
+		gchar action_name[80];
+		gchar accelerator[80];
+
+		snprintf(action_name, sizeof (action_name), "Tab%d", i);
+		action = gtk_action_new (action_name, NULL, NULL, NULL);
+		g_object_set_data (G_OBJECT (action), "num", GINT_TO_POINTER (i));
+		g_signal_connect (action, "activate",
+				G_CALLBACK (action_tab_change_action_activate_callback), window);
+		snprintf(accelerator, sizeof (accelerator), "<alt>%d", (i+1)%10);
+		gtk_action_group_add_action_with_accel (action_group, action, accelerator);
+		g_object_unref (action);
+		gtk_ui_manager_add_ui (ui_manager,
+				gtk_ui_manager_new_merge_id (ui_manager),
+				"/",
+				action_name,
+				action_name,
+				GTK_UI_MANAGER_ACCELERATOR,
+				FALSE);
+
+	}
+
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	g_object_unref (action_group); /* owned by ui_manager */
+
 	gtk_window_add_accel_group (GTK_WINDOW (window),
 				    gtk_ui_manager_get_accel_group (ui_manager));
 	
@@ -899,14 +1306,13 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 			  G_CALLBACK (connect_proxy_cb), window);
 	g_signal_connect (ui_manager, "disconnect_proxy",
 			  G_CALLBACK (disconnect_proxy_cb), window);
-	
-	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
-	g_object_unref (action_group); /* owned by ui manager */
 
+	/* add the UI */
 	ui = nautilus_ui_string_get ("nautilus-shell-ui.xml");
 	gtk_ui_manager_add_ui_from_string (ui_manager, ui, -1, NULL);
 
 	nautilus_window_initialize_trash_icon_monitor (window);
+	nautilus_window_initialize_go_menu (window);
 }
 
 static GList *
